/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Rodger (Jieliang) Luo                        *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization.                         *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;

// 2D matrix table related variables
Table table;
int numRows, numColumns;
float[][] dataMatrix;
float maxValue, minValue;

// width and height for each cell
float cellWidth, cellHeight;

// text labels
int startYear = 2006;

void setup() {
  size(1280, 720); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  // Make the screen resizable.                     
  surface.setResizable(true);

  //Put data into Processing table from csv
  table = loadTable("top100Dewey.csv", "header");
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  dataMatrix = new float[numColumns-1][numRows];
  println("Rows: " + numRows + " Columns: " + numColumns);

  for (int i=0; i<numColumns-1; i++) {
    for (int j=0; j<numRows; j++) {
      dataMatrix[i][j] = table.getFloat(j, i+1);
    }
  }

  maxValue = max2D(dataMatrix);
  minValue = min2D(dataMatrix);

  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
}

void draw() {

  //refresh the canvas every frame
  background(250);

  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/numRows;
  cellHeight = (height - constantVerticalMargin*2)/(numColumns-1);

  //Draw 2D Matrix
  for (int i=0; i<numColumns-1; i++) {
    for (int j=0; j<numRows; j++) {
      stroke(255);
      strokeWeight(1);
      float greyScale = map(dataMatrix[i][j], minValue, maxValue, 255, 0);
      fill(greyScale);
      rect(constantHorizontalMargin + cellWidth*j, constantVerticalMargin 
        + cellHeight*i, cellWidth, cellHeight);
    }
  }

  //Draw vertical labels
  for (int i=0; i<numColumns-1; i++) {
    textAlign(RIGHT, CENTER);
    fill(0);
    textSize(10);
    text(startYear+i, constantHorizontalMargin, 
      constantVerticalMargin + cellHeight/2 + cellHeight*i);
  }

  //Draw Title
  textAlign(CENTER, CENTER);
  textSize(16);
  text("Top 100 Active DEWEY CATEGORIES FROM 2006 TO 2014", width/2, constantVerticalMargin/2);
}